# -*- coding: utf-8 -*-

# Authors: Alejandro J. Cura <alecu@canonical.com>
#          Roberto Alsina <roberto.alsina@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the Qt UI."""

from PyQt4 import QtCore

from ubuntuone.credentials import (
    APP_NAME,
    TC_URL,
    PING_URL,
    DESCRIPTION,
)

import ubuntu_sso

from ubuntuone_installer.gui.qt import gui
from ubuntuone_installer.gui.qt.tests import BaseTestCase
from ubuntuone_installer.gui.qt.embedded_sso import UbuntuSSOClientGUI

TOKEN = {u'consumer_key': u'xQ7xDAz',
         u'consumer_secret': u'KzCJWCTNbbntwfyCKKjomJDzlgqxLy',
         u'token_name': u'test',
         u'token': u'GkInOfSMGwTXAUoVQwLUoPxElEEUdhsLVNTPhxHJDUIeHCPNEo',
         u'token_secret': u'qFYImEtlczPbsCnYyuwLoPDlPEnvNcIktZphPQklAWrvyfFMV'}


class FakeCredentials(object):
    """A very dummy Credentials object."""

    def __init__(self, *a, **kw):
        self.args = a
        self.kwargs = kw


class FakeController(object):
    """A fake controller for the tests."""

    def __init__(self, *args, **kwargs):
        pass

    # pylint: disable=C0103
    def setupUi(self, view):
        """Fake the setup."""
    # pylint: enable=C0103
    is_fake = True


class MainWindowTestCase(BaseTestCase):
    """Test the qt main window."""

    class_ui = gui.MainWindow

    def setUp(self):
        """Initialize this test instance."""
        # Faking each SSO object instead of doing it lower
        # so we don't rely on any SSO behaviour
        self.patch(gui, "Credentials", FakeCredentials)
        self.patch(gui, "SetUpAccountController", FakeController)
        self.patch(gui, "TosController", FakeController)
        self.patch(gui, "EmailVerificationController", FakeController)
        self.patch(gui, "CurrentUserController", FakeController)
        self.patch(gui, "SuccessController", FakeController)
        self.patch(gui, "ForgottenPasswordController", FakeController)
        self.patch(gui, "ResetPasswordController", FakeController)
        self.patch(gui, "ResetPasswordController", FakeController)
        super(MainWindowTestCase, self).setUp()

    def test_done_calls_custom_close_callback(self):
        """When closing the window, close_callback is called."""
        self.ui.close_callback = self._set_called
        self.ui.done(result=0)
        self.assertEqual(self._called,
                         ((), {}), 'close_callback called.')

    def test_close_callback_can_be_none(self):
        """The close_callback can be None."""
        self.ui.close_callback = None
        self.ui.done(result=0)
        # world did not explode

    def test_start_control_panel_on_finishing(self):
        """If done is called with result=1, the control panel
        should be called."""
        self.patch(gui.subprocess, "Popen", self._set_called)
        self.ui.done(result=1)
        self.assertEqual(self._called,
            ((['ubuntuone-control-panel-qt'],), {}))

    def test_not_start_control_panel_on_cancel(self):
        """If done is called with result=0, the control panel
        should NOT be called."""
        self.patch(gui.subprocess, "Popen", self._set_called)
        self.ui.done(result=0)
        self.assertEqual(self._called, False)

    def test_credential_parameters(self):
        """Test if the credentials are created the same way
        as in control panel."""
        self.assertEqual(self.ui.creds.kwargs, {
            'app_name': APP_NAME,
            'ui_module': "ubuntuone_installer.gui.qt.embedded_sso",
            'tc_url': TC_URL,
            'ping_url': PING_URL,
        })


class SSOGuiTestCase(BaseTestCase):
    """Test the custom SSO GUI."""
                
    class_ui = UbuntuSSOClientGUI

    def setUp(self):
        """Initialize this test instance."""

        self.kwargs = {
            'app_name' : APP_NAME,
            'tc_url' : TC_URL,
            'help_text' : '',
        }
        QtCore.QCoreApplication.instance().window = True
        super(SSOGuiTestCase, self).setUp()

    def test_sso_client_gui(self):
        """Ensure the class stores the right parameters."""
        
        self.assertEqual( isinstance(
            self.ui.controller, 
            ubuntu_sso.qt.controllers.UbuntuSSOWizardController), True)
        self.assertEqual(self.ui.view, True)
        